# encoding: utf-8

import couchdb
from optparse import make_option
from hudjango.management.commands.couchdbcommand import CouchDBCommand
from django.core.management.base import CommandError


class Command(CouchDBCommand):
    help = """Creates a new couchdb database."""
    option_list = CouchDBCommand.option_list + (
        make_option('--purge', '-p', action='store_true', help='Delete existing database [default: %default]'),
    )
    
    def handle(self, *args, **options):
        if len(args) != 1:
            raise CommandError("You need to specify exactly one argument as database name")
        database = args[0]
        server = couchdb.client.Server(options['server'])

        if options['purge']:
            try:
                server.delete(database)
            except couchdb.client.ResourceNotFound:
                pass
        
        try:
            server.create(database)
        except couchdb.client.PreconditionFailed, exception:
            raise CommandError("%s: %s" % (database, exception.message[1]))
        print "database '%s' created succesfully" % database




# encoding: utf-8
"""
Created by Johan Otten on 2009-10-12.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

from datetime import datetime

from django.test import TestCase
from django.contrib.admin.models import CHANGE, ADDITION
from django.contrib.auth.models import User

from hudjango import log_action
from regressiontests.serializers.models import Place


class TestLog(TestCase):
    """Test the logaction wrapper function."""

    def setUp(self):
        """Create a entry in the database."""
        self.place1 = Place.objects.create(name="Dempsey's", address="623 Vermont St",
                                      last_visit=datetime(2008, 7, 6, 5, 4, 3))
        self.place2 = Place.objects.create(name="Eich", address=u"An dür Eich 1",
                                      last_visit=datetime(2008, 7, 6, 5, 4, 3))

        self.user = User.objects.create_user('logger', 'lennon@example.com', 'logger')
        self.user2 = User.objects.create_user('logger2', 'lennon@example.com', 'logger2')


    def test_log_action(self):
        """Test the log_action function."""
        # test the most simplest form.
        log_action(self.place1, CHANGE)

        log_action(self.place2, ADDITION, self.user2)

        log_action(self.place2, ADDITION, self.user2, "this is a message")

        log_action(self.place2, ADDITION, self.user2, "this is a message", u"this is the repr")




#!/usr/bin/env python
# encoding: utf-8
"""
decorators.py

Created by Christian Klein on 2010-03-26.
Copyright (c) 2007 HUDORA GmbH. All rights reserved.
"""

from functools import wraps
from django.http import HttpResponse
import simplejson as json


# The ajax_request decorator was taken from django-annoying
# see http://pypi.python.org/pypi/django-annoying/0.7.4

class JsonResponse(HttpResponse):
    """
    HttpResponse descendant, which return response with ``application/json`` mimetype.
    """
    def __init__(self, data):
        super(JsonResponse, self).__init__(content=json.dumps(data), mimetype='application/json')


def ajax_request(func):
    """
    If view returned serializable dict, returns JsonResponse with this dict as content.

    example:

        @ajax_request
        def my_view(request):
            news = News.objects.all()
            news_titles = [entry.title for entry in news]
            return {'news_titles': news_titles}
    """
    @wraps(func)
    def wrapper(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        if isinstance(response, dict):
            return JsonResponse(response)
        else:
            return response
    return wrapper



#testscalingimagefield.py - tests for scalingimagefield.py

import os
import re
import unittest
from random import random
    
from django.conf import settings
from django.dispatch import dispatcher
import hudjango.fields.scalingimagefield as sif

axb = re.compile(r"\d+x\d+")


class emptyo(object):
    "Just sits there to have attributes assigned to it"
    pass


class image(object):
    """Test image."""

    def __init__(self, x, y):
        self.size = (x, y)

    def resize(self, size, quality=1):
        self.size = size
        return self

    def crop(self, dimensions):
        leftx, lefty, rightx, righty = dimensions
        self.size = (rightx - leftx, righty - lefty)
        self.cropDims = dimensions
        return self


class TestSIF(unittest.TestCase):

    def setUp(self):
        self.bigimage = image(1280, 1024)
        self.smallimage = image(640, 480)
    
    def testModuleVars(self):
        for k, v in sif._sizes.iteritems():
            self.assert_(k.isalpha())
            self.assert_(axb.match(v))
    
    def test_ScaleImageConstrainY(self):
        self.assertEqual(sif._scaleImage(500, 300, self.bigimage).size, (375, 300))

    def test_scaleImageConstrainX(self):
        self.assertEqual(sif._scaleImage(300, 500, self.bigimage).size, (300, 240))
    
    def test_scaleImageGrow(self):
        self.assertEqual(sif._scaleImage(800, 600, self.smallimage).size, (640, 480))
    
    def test_cropImageTooBig(self):
        self.assertEqual(sif._cropImage(1280, 1024, self.smallimage).cropDims, (0, 0, 640, 480))
    
    def test_cropImageConstrainX(self):
        self.assertEqual(sif._cropImage(1024, 1024, self.bigimage).cropDims, (128, 0, 1152, 1024))
    
    def test_cropImageConstrainY(self):
        self.assertEqual(sif._cropImage(1280, 1000, self.bigimage).cropDims, (0, 12, 1280, 1012))


class TestImageScaler(unittest.TestCase):

    def setUp(self):
        self.path = str(random())
        field = emptyo()
        field.attname = "image"
        parent = emptyo()
        parent.image = self.path
        self.imagescaler = sif.Imagescaler(field, parent)
    
    def testImageScalerInit(self):
        self.assert_(isinstance(self.imagescaler, sif.Imagescaler))
    
    def testImageScalerAttrs(self):
        self.assertEqual(self.path, self.imagescaler.original_image)
        self.assertEqual(os.path.join(settings.MEDIA_ROOT, self.path), self.imagescaler.original_image_path)
        
        for size in sif._sizes:
            for end in ['', '_path', '_dimensions', '_tag']:
                self.assert_(size + end in dir(self.imagescaler))
                self.assert_(hasattr(getattr(self.imagescaler, size + end), '__call__'))
    
    def testImagescalerScaledFilename(self):
        self.assertEquals(self.imagescaler.scaled_filename(), 'broken.gif')
    
    def testImageScalerScaledURL(self):
        self.assertEquals(self.imagescaler.scaled_url(), 'broken.gif')
    
    def testImageScalerScaledDimensions(self):
        "broken.gif doesn't actually exist"
        self.assertRaises(IOError, self.imagescaler.scaled_dimensions)
    

class TestScalingImageField(unittest.TestCase):

    def setUp(self):
        self.sif = sif.ScalingImageField()
        
        self.__class__._meta = emptyo()

        def add_field(f):
            pass

        self.__class__._meta.add_field = add_field
        self.__class__._meta.module_name = "testscalingimagefield"
        self.__class__._meta.object_name = "TestScalingImageField"
        self.__class__._get_FIELD_filename = None
        self.__class__._get_FIELD_url = None
        self.__class__._get_FIELD_size = None
        self.__class__._get_FIELD_width = None
        self.__class__._get_FIELD_height = None
    
    def testSIFInit(self):
        self.assert_(isinstance(self.sif, sif.ScalingImageField))
    
    def testSIFinternaltype(self):
        self.assertEqual(self.sif.get_internal_type(), "FileField")
    
classes = [TestSIF, TestImageScaler, TestScalingImageField]
suite = unittest.TestSuite([unittest.TestLoader().loadTestsFromTestCase(cls) for cls in classes])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
Django Template Tags

Created by Maximillian Dornseif on 2006-08-22.
Copyright (c) 2006-2008 HUDORA GmbH. Consider it BSD licensed.
"""

import operator
import re
import urllib
from django import template
from django.template import resolve_variable
from django.utils.html import conditional_escape
from django.utils.text import smart_split
from django.utils.safestring import mark_safe
huimages = None # lazy import, see ImageLink.__init__()


register = template.Library()


# TODO: move to myplfrontend
@register.filter
def format_location(value, dummy):
    """
    Formats a myPL location nicely.
    """
    
    # TODO: move to myPLfrontend - this is not of general interest.
    if len(value) == 6 and str(value).isdigit():
        return "%s-%s-%s" % (value[:2], value[2:4], value[4:])
    return value


@register.filter
def slashencode(value):
    """
    Like Django's urlencode but also encodes slashes.
    Example::
    
        {% 14600/WK|slashencode %} results in 14600%2FWK;
    """
    
    return urllib.quote(value, '')


@register.filter
def format_tel(obj, autoescape=None):
    """Formats a Phone Number as tel: URI."""
    if autoescape:
        esc = conditional_escape
    else:
        esc = lambda x: x
    cleantel = re.sub('[^0-9+]+', '', str(obj))
    if cleantel.strartswith('+'):
        cleantel = "00%s" % cleantel.replace('+', '')
    return mark_safe('<a href="tel:%s">%s</span>' % (esc(cleantel), esc(obj)))
format_tel.needs_autoescape = True


@register.filter
def format_address(obj, autoescape=None):
    """Formats the address in a Lieferung/Lieferschein/Kunde as nice XHTML.

    The obj must follow the address-protocol outlined at
    http://cybernetics.hudora.biz/projects/wiki/AddressProtocol

    The generated XHTML follows hCard described at http://microformats.org/wiki/hcard
    """

    ret = []
    if autoescape:
        esc = conditional_escape
    else:
        esc = lambda x: x

    kdnstr = '<span class="org name1">%s</span>' % esc(obj.name1)
    if hasattr(obj, 'kundennr') and obj.kundennr:
        kdnstr += ' (<span class="customerid">%s</span>)' % esc(obj.kundennr)
    if hasattr(obj, 'kundennummer') and obj.kundennummer: # TODO: remove support for kundennummer (legacy)
        kdnstr += ' (<span class="customerid">%s</span>)' % esc(obj.kundennummer)
    ret.append(kdnstr)
    for dataname in ['name2', 'name3']:
        data = ''
        if hasattr(obj, dataname):
            data = getattr(obj, dataname)
        if data:
            ret.append('<span class="%s">%s</span>' % (dataname, esc(data)))
    if hasattr(obj, 'iln') and obj.iln:
        kdnstr += 'ILN <span class="iln">%s</span>' % esc(obj.iln)
    ret.append('<span class="adr">')
    for dataname in ['adresse', 'address', 'street', 'strasse']:
        data = ''
        if hasattr(obj, dataname):
            data = getattr(obj, dataname)
        if data:
            ret.append('<span class="%s">%s</span>' % (dataname, esc(data)))
    if hasattr(obj, 'plz'):
        ortstr = ('<span class="zip postal-code">%s</span>'
                  ' <span class="city locality">%s</span>' % (esc(obj.plz), esc(obj.ort)))
    else:
        ortstr = ('<span class="zip postal-code">%s</span>'
                  ' <span class="city locality">%s</span>' % (esc(obj.postleitzahl), esc(obj.ort)))
    land = 'DE'
    if hasattr(obj, 'land'):
        land = obj.land
    else:
        land = obj.laenderkennzeichen
    if land != 'DE':
        ortstr = ('<span class="country-name land">%s</span>-' % esc(land)) + ortstr
    ret.append(ortstr)
    ret.append('</span>')
    # the whole thing is enclesed in a vcard class tag
    return mark_safe('<div class="address vcard">%s</div>' % '<br/>'.join(ret))
format_address.needs_autoescape = True


def _get_attr(obj, key):
    """Get an object attribute or an dict key"""
    ret = getattr(obj, key, '')
    if (not ret) and hasattr(obj, 'get'):
        ret = obj.get(key, '')
    return ret


@register.filter
def format_addressproto(obj, autoescape=None):
    """Formats the address as nice XHTML. Includes telephone number and the like.
    
    The obj must follow the address-protocol outlined at
    http://github.com/hudora/huTools/blob/master/doc/standards/address_protocol.markdown
    
    The generated XHTML follows hCard described at http://microformats.org/wiki/hcard
    
    Works with objects:
    >>> class Dummy(object): pass
    >>> addr = Dummy()
    >>> addr.name1 = 'nameTEST'
    >>> format_addressproto(addr)
    '<div class="address vcard"><span class="org name1">nameTEST</span><br/><span class="adr"><span class="zip postal-code"></span> <span class="city locality"></span></span></div>'
    
    Works also with dicts:
    >>> format_addressproto({'name1': 'TESTname', 'ort': 'Rade', 'plz': '42477'})
    '<div class="address vcard"><span class="org name1">TESTname</span><br/><span class="adr"><span class="zip postal-code">42477</span> <span class="city locality">Rade</span></span></div>'
    """
    # it also supports some legacy variants of the address-protocol
    
    ret = []
    if autoescape:
        esc = conditional_escape
    else:
        esc = lambda x: x
    
    kdnstr = '<span class="org name1">%s</span>' % esc(_get_attr(obj, 'name1'))
    if _get_attr(obj, 'kundennr'):
        kdnstr += ' (<span class="customerid">%s</span>)' % esc(_get_attr(obj, 'kundennr'))
    elif _get_attr(obj, 'softmid'):
        kdnstr += ' (<span class="customerid">SC%s</span>)' % esc(_get_attr(obj, 'softmid'))
    elif _get_attr(obj, 'curmid'):
        kdnstr += ' (<span class="customerid">%s</span>)' % esc(_get_attr(obj, 'curmid'))
    ret.append(kdnstr)

    for dataname in ['name2', 'name3']:
        data = _get_attr(obj, dataname)
        if data:
            ret.append('<span class="%s">%s</span>' % (dataname, esc(data)))
    
    if _get_attr(obj, 'iln'):
        kdnstr += 'ILN <span class="iln">%s</span>' % esc(_get_attr(obj, 'iln'))
    
    addr = []
    for dataname in ['adresse', 'address', 'street', 'strasse']:
        data = _get_attr(obj, dataname)
        if data:
            addr.append('<span class="%s">%s</span>' % (dataname, esc(data)))
    if _get_attr(obj, 'plz'):
        ortstr = ('<span class="zip postal-code">%s</span>'
                  ' <span class="city locality">%s</span>' % (esc(_get_attr(obj, 'plz')),
                                                              esc(_get_attr(obj, 'ort'))))
    else:
        ortstr = ('<span class="zip postal-code">%s</span>'
                  ' <span class="city locality">%s</span>' % (esc(_get_attr(obj, 'postleitzahl')),
                                                              esc(_get_attr(obj, 'ort'))))
    land = _get_attr(obj, 'land')
    if not land:
        land = _get_attr(obj, 'laenderkennzeichen')
    if not land:
        land = 'DE'
    if land != 'DE':
        ortstr = ('<span class="country-name land">%s</span>-' % esc(land)) + ortstr
    addr.append(ortstr)
    ret.append('<span class="adr">%s</span>' % '<br/>'.join(addr))
    
    if _get_attr(obj, 'tel'):
        ret.append('<span class="tel"><span class="type">main</span>:<span class="value">%s</span></span>'
                   % esc(_get_attr(obj, 'tel')))
    if _get_attr(obj, 'mobile'):
        ret.append('<span class="tel"><span class="type">mobile</span>:<span class="value">%s</span></span>'
                   % esc(_get_attr(obj, 'mobile')))
    if _get_attr(obj, 'fax'):
        ret.append('<span class="tel"><span class="type">fax</span>:<span class="value">%s</span></span>'
                   % esc(_get_attr(obj, 'fax')))
    if _get_attr(obj, 'email'):
        ret.append('<span class="email">%s</span>' % esc(_get_attr(obj, 'email')))
    
    # the whole thing is enclesed in a vcard class tag
    return mark_safe('<div class="address vcard">%s</div>' % '<br/>'.join(ret))
format_addressproto.needs_autoescape = True


@register.filter
def deumlaut(value):
    """
    Converts Umlauts to ae, oe, etc.
    
    Example::
    
        {% Würstchen|deumlaut %} results in Wuerstchen
    """
    
    # TODO: use huTools.unicode.deUmlaut()
    value = value.replace('ß', 'ss').replace('ö', 'oe').replace('ä', 'ae').replace('ü', 'ue')
    return value.replace('Ö', 'Oe').replace('Ä', 'Ae').replace('Ü', 'ue')


@register.filter
def html_euro(value):
    """
    Formats a value with two decimal digits and adds an Euro sign in HTML entity notation.
    
    Example::
    
        {% 1.2345|html_euro %} results in 1.23&nbsp;&euro;
    """
    
    try:
        value = float(value)
    except ValueError:
        return value
    except TypeError:
        return value
    return mark_safe('%.2f&nbsp;&euro;' % (value))


@register.filter
def html_cent(value):
    """
    Formats a value with up to 5 decimal digits and andds an cent sign in HTML entity notation.
    
    Example::
    
        {% 0.00012345|html_cent %} results in 0.00012&nbsp;&cent;
    """
    
    try:
        value = float(value)
    except ValueError:
        return value
    except TypeError:
        return value
    return mark_safe('%.5f&nbsp;&cent;' % (value))


@register.filter
def g2kg(value):
    """
    Convert Gramms to kg by dividing by 1000.
    
    Example::
    
        {% 1000|g2kg %} results in 1
        {% 422|g2kg %} results in 0.4
    """
    
    try:
        value = int(value)
    except ValueError:
        return value
    except TypeError:
        return value
    if value < 100:
        return mark_safe('0.1')
    if value < 1000:
        return mark_safe('0.%d' % int(value/100))
    return mark_safe('%d' % int(value/1000))
    


def _cond_helper(func, arguments):
    """
    Helper function for the conditional filters.
    
    The arguments are converted to float numbers before
    the comparison function is applied.
    
    >>> _cond_helper(operator.gt, (0, 1))
    False
    
    >>> _cond_helper(operator.gt, (1, 0))
    True
    
    >>> _cond_helper(operator.lt, ("1", 2.3))
    True
    
    >>> _cond_helper(operator.le, ("abc", 0))
    False
    
    """
    try:
        numbers = [float(tmp) for tmp in arguments]
        return func(*numbers)
    except ValueError:
        return False
    

@register.filter(name='lt')
def lt_filter(value, arg):
    """Return if the float representation of value is less than the float representation of arg."""
    return _cond_helper(operator.lt, (value, arg))
    

@register.filter(name='le')
def le_filter(value, arg):
    """Return if the float representation of value is less than or equal to arg."""
    return _cond_helper(operator.le, (value, arg))
    

@register.filter(name='gt')
def gt_filter(value, arg):
    """Return if the float representation of value is greater than the float representation of arg."""
    return _cond_helper(operator.gt, (value, arg))
    

@register.filter(name='ge')
def ge_filter(value, arg):
    """Return if the float representation of value is greater than or equal to arg."""
    return _cond_helper(operator.ge, (value, arg))


class LinkObject(template.Node):
    """Helper class for do_link_object()."""
    
    def __init__(self, obj):
        super(LinkObject, self).__init__()
        self.obj = obj
    
    def render(self, context):
        """Called when the page is actually rendered."""
        obj = resolve_variable(self.obj, context)
        if hasattr(obj, 'get_absolute_url'):
            return '<a href="%s">%s</a>' % (obj.get_absolute_url(), unicode(obj))
        return unicode(obj)
    

def do_link_object(dummy, token):
    """
    Create a Link to a Database Object.
    
    This uses an object's get_absolute_url() and __unicode__() methods to generate a link tag-pair to this
    object by basically transforming 
    
    Example::
    
        {% link_object %} to:
        <a href="obj.get_absolute_url()">str(obj)</a>
    
    If the object doesn't have a get_absolute_url() we fall back to just returning the string representation
    of the object.
    """
    try:
        # split_contents() knows not to split quoted strings.
        dummy, obj = token.contents.split()
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires a single argument" % token.contents[0]
    return LinkObject(obj)
register.tag('link_object', do_link_object)


class PrinterChoiceNode(template.Node):
    
    def __init__(self):
        self.current_printer = template.Variable('printer_choice')
        self.printer_list = template.Variable('printer_choice_list')
    
    def render(self, context):
        try:
            current_printer = self.current_printer.resolve(context)
        except template.VariableDoesNotExist:
            current_printer = None
        try:
            printer_list = self.printer_list.resolve(context)
        except template.VariableDoesNotExist:
            printer_list = ['NullPrinter']
        options = []
        for printer in printer_list:
            if printer == current_printer:
                options.append('<option value="%s" selected>%s</option>' % (printer, printer))
            else:
                options.append('<option value="%s">%s</option>' % (printer, printer))
        return '<select name="printer_choice">%s</select>' % ('\n'.join(options))


@register.tag
def printer_choice(dummy, token):
    """Display a simple printer choice dialog.
    
    This is meant to be used in conjunction with hudjango.PrinterChooser. It expects the 
    variables printer_choice and printer_choice_list to be available in the context.
    PrinterChooser.update_context can do that automatically. For futher information see the
    documentation of hudjango.PrinterChooser."""
    
    try:
        dummy = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("%r tag requires no arguments" % token.contents.split()[0])
    return PrinterChoiceNode()


class ImageLink(template.Node):
    """Helper class for rendering do_imageid()."""
    
    def __init__(self, obj, options):
        # lazy import
        global huimages
        if huimages is None:
            import huimages
        super(ImageLink, self).__init__()
        self.obj = obj
        self.options = options
    
    def render(self, context):
        """Called when the page is actually rendered."""
        obj = resolve_variable(self.obj, context)
        imageid = getattr(obj, 'name', obj)
        if not imageid:
            return ''
        if self.options.get('urlonly', False):
            return unicode(huimages.imageurl(imageid))
        else:
            if self.options.get('nolink', False):
                return huimages.scaled_tag(imageid, self.options.get('size', '1024x768'))
            else:
                return unicode('<a class="imagelink" href="%s">%s</a>' % (huimages.imageurl(imageid),
                    huimages.scaled_tag(imageid, self.options.get('size', '1024x768'))))
    

def do_imageid(dummy, token):
    """Renders an Imagetag by ImageID. See pydoc huimages for information about ImageID.
       
       Syntax: {% imageid <imageid> <size> [nolink] [urlonly] %}
       
       See huimages for further Documentation on size.
       
       Example::
       
           {% imageid spare.bild "320x240" %} 
           <a class="imagelink" href="http://i.hdimg.net/o/TZGBY4PX2BXMKXXGNMYHZBVYZXSJBOLT01.jpeg"><img src="http://i.hdimg.net/320x240/TZGBY4PX2BXMKXXGNMYHZBVYZXSJBOLT01.jpeg" /></a>
           
           {% imageid "DEQOQMIPRJPPTAKLE3NQY5BTMMNTFGG201" "320x240" nolink %} 
           <img src="http://i.hdimg.net/320x240/DEQOQMIPRJPPTAKLE3NQY5BTMMNTFGG201.jpeg" />
           
           {% imageid spare.bild.name "150x150!" urlonly %} 
           http://i.hdimg.net/150x150!/TZGBY4PX2BXMKXXGNMYHZBVYZXSJBOLT01.jpeg
    """
    options = {'urlonly': False, 'nolink': False, 'size': '1024x768'}
    try:
        tokens = list(smart_split(token.contents))
        dummy, obj = tokens[:2]
        if len(tokens) > 2:
            options['size'] = tokens[2].strip('"\'')
        for option in tokens[2:]:
            if option.strip('"\'') == 'urlonly':
                options['urlonly'] = True
            if option.strip('"\'') == 'nolink':
                options['nolink'] = True
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires a single argument" % token.contents.split()[0]
    return ImageLink(obj, options)
register.tag('imageid', do_imageid)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#testdefaulting - unit tests for defaulting.py

import unittest
import defaulting
from random import random
from django.dispatch import dispatcher


class emptyo(object):
    "Just sits there to have attributes assigned to it"
    pass


class TestDefaulting(unittest.TestCase):
    """Test Defaultin."""

    def setUp(self):
        """Create a few DefaultingCharFields."""

        self.dcfValue = random()
        self.dcf = defaulting.DefaultingCharField(self.dcfValue)
        self.dcf2 = defaulting.DefaultingCharField()
        
        self.__class__._meta = emptyo()

        def add_field(f):
            pass

        self.__class__._meta.add_field = add_field
        self.__class__._meta.module_name = "testaudit"
        self.__class__._meta.object_name = "TestUpdatedByField"
    
    def testInit(self):
        self.assertEqual(self.dcfValue, self.dcf.default_from_field)
        self.assert_(self.dcf2.default_from_field is None)
        self.assert_(isinstance(self.dcf, defaulting.DefaultingCharField))
    
    def testInternalType(self):
        self.assertEqual("CharField", self.dcf.get_internal_type())
    

suite = unittest.TestLoader().loadTestsFromTestCase(TestDefaulting)

if __name__ == '__main__':
    unittest.main()



# based on http://code.djangoproject.com/wiki/CookBookThreadlocalsAndUser
# http://lukeplant.me.uk/blog.php?id=1107301634
# http://svn.zyons.python-hosting.com/trunk/zilbo/common/utils/middleware/threadlocals.py
# slightly modidies by Maximillian Dornseif, September 2006

"""Hand request data down to non-request related functions.

Usually with request related information like remote IP address or logged  in user you are unable to access
this information in non-request related modules of the django stack, e.g. in models.
Handing down this information is somewhat tricky since we have to keep multi-threaded webservers in mind.

With this middleware you can use things like UpdatedByField and CreatedByIPField.
"""

# threadlocals middleware
try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

_thread_locals = local()


def set_current_user(user):
    """Stores the logged in user serviced by this thread"""
    _thread_locals.user = user


def get_current_user():
    """query the information stored by ThreadLocals."""
    return getattr(_thread_locals, 'user', None)


def set_remote_ip(ipaddr):
    """Stores the remote IP address serviced by this thread"""
    _thread_locals.remote_ip = ipaddr


def get_remote_ip():
    """query the information stored by ThreadLocals."""
    return getattr(_thread_locals, 'remote_ip', None)


class ThreadLocals(object):
    """Middleware that gets various objects from the
    request object and saves them in thread local storage."""

    def process_request(self, request):
        """Store certain data at every request."""
        set_current_user(getattr(request, 'user', None))
        set_remote_ip(request.META.get('REMOTE_ADDR', None))



"""Extended fieldtypes for use with django.

defaulting        - set default values
scalingimagefield - ImageField variant  wich scales images on the fly.
"""

from hudjango.fields.defaulting import *
from hudjango.fields.scalingimagefield import *
from django.db.models import ManyToManyField


class RelaxedManyToManyField(ManyToManyField):
    """"This is a many to many field which can handle non integer primary keys for the relation."""
    
    def isValidIDList(self, field_data, all_data):
        "Validates that the value is a valid list of foreign keys"
        mod = self.rel.to
        pks = field_data.split(',')
        objects = mod._default_manager.in_bulk(pks)
        if len(objects) != len(pks):
            badkeys = [k for k in pks if k not in objects]
            raise validators.ValidationError, "Please enter valid %(self)s IDs." % {'self': self.verbose_name}



#!/usr/bin/env python

import figleaf
figleaf.start(ignore_python_lib=True)

import doctest
import os
import sys
import traceback
import unittest

REGRESSION_TESTS_DIR_NAME = 'regressiontests'

TEST_TEMPLATE_DIR = 'templates'

REGRESSION_TEST_DIR = os.path.join(os.path.dirname(__file__), REGRESSION_TESTS_DIR_NAME)

ALWAYS_INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.sites',
    #'django.contrib.flatpages',
    #'django.contrib.redirects',
    'django.contrib.sessions',
    #'django.contrib.comments',
    'django.contrib.admin',
]


def get_test_models():
    models = []
    for loc, dirpath in [(REGRESSION_TESTS_DIR_NAME, REGRESSION_TEST_DIR), ]:
        for f in os.listdir(dirpath):
            if f.startswith('__init__') or f.startswith('.') or f.startswith('sql') \
                or f.startswith('invalid') or f.endswith('.py') or f.endswith('.pyc'):
                continue
            models.append((loc, f))
    return models
    

def django_tests(verbosity, interactive, test_labels):
    from django.conf import settings
    
    old_installed_apps = settings.INSTALLED_APPS
    old_test_database_name = settings.TEST_DATABASE_NAME
    old_root_urlconf = settings.ROOT_URLCONF
    old_template_dirs = settings.TEMPLATE_DIRS
    old_use_i18n = settings.USE_I18N
    old_login_url = settings.LOGIN_URL
    old_language_code = settings.LANGUAGE_CODE
    old_middleware_classes = settings.MIDDLEWARE_CLASSES

    # Redirect some settings for the duration of these tests.
    settings.INSTALLED_APPS = ALWAYS_INSTALLED_APPS
    settings.ROOT_URLCONF = 'urls'
    settings.TEMPLATE_DIRS = (os.path.join(os.path.dirname(__file__), TEST_TEMPLATE_DIR), )
    settings.USE_I18N = True
    settings.LANGUAGE_CODE = 'en'
    settings.LOGIN_URL = '/accounts/login/'
    settings.MIDDLEWARE_CLASSES = (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.middleware.common.CommonMiddleware',
    )
    settings.SITE_ID = 1

    # Load all the ALWAYS_INSTALLED_APPS.
    # (This import statement is intentionally delayed until after we
    # access settings because of the USE_I18N dependency.)
    from django.db.models.loading import get_apps, load_app
    get_apps()

    # Load all the test model apps.
    test_models = []
    for model_dir, model_name in get_test_models():
        model_label = '.'.join([model_dir, model_name])
        try:
            # if the model was named on the command line, or
            # no models were named (i.e., run all), import
            # this model and add it to the list to test.
            if not test_labels or model_name in set([label.split('.')[0] for label in test_labels]):
                if verbosity >= 1:
                    print "Importing model %s" % model_name
                mod = load_app(model_label)
                if mod:
                    if model_label not in settings.INSTALLED_APPS:
                        settings.INSTALLED_APPS.append(model_label)
        except Exception, e:
            sys.stderr.write("Error while importing %s:" % model_name + ''.join(traceback.format_exception(*sys.exc_info())[1:]))
            continue
    
    # Run the test suite, including the extra validation tests.
    from django.test.simple import run_tests
    failures = run_tests(test_labels, verbosity=verbosity, interactive=interactive)
    # now test docstrings in modules directly without the test runner
    for name, mod in sys.modules.items():
        if name.startswith('hudjango'):
            failure_count, test_count = doctest.testmod(mod)
            failures += failure_count
    
    if failures:
        sys.exit(failures)
    
    # Restore the old settings.
    settings.INSTALLED_APPS = old_installed_apps
    settings.ROOT_URLCONF = old_root_urlconf
    settings.TEMPLATE_DIRS = old_template_dirs
    settings.USE_I18N = old_use_i18n
    settings.LANGUAGE_CODE = old_language_code
    settings.LOGIN_URL = old_login_url
    settings.MIDDLEWARE_CLASSES = old_middleware_classes
    

if __name__ == "__main__":
    from optparse import OptionParser
    usage = "%prog [options] [model model model ...]"
    parser = OptionParser(usage=usage)
    parser.add_option('-v', '--verbosity', action='store', dest='verbosity', default='0',
        type='choice', choices=['0', '1', '2'],
        help='Verbosity level; 0=minimal output, 1=normal output, 2=all output')
    parser.add_option('--noinput', action='store_false', dest='interactive', default=True,
        help='Tells Django to NOT prompt the user for input of any kind.')
    parser.add_option('--settings',
        help='Python path to settings module, e.g. "myproject.settings". If this isn\'t provided, the DJANGO_SETTINGS_MODULE environment variable will be used.')
    options, args = parser.parse_args()
    if options.settings:
        os.environ['DJANGO_SETTINGS_MODULE'] = options.settings
    elif "DJANGO_SETTINGS_MODULE" not in os.environ:
        parser.error("DJANGO_SETTINGS_MODULE is not set in the environment. "
                      "Set it or use --settings.")
    django_tests(int(options.verbosity), options.interactive, args)
    figleaf.stop()
    figleaf.write_coverage('.figleaf', append=True)



"""
This is a Custom Storage System for Django with CouchDB backend.
Created by Christian Klein.
(c) Copyright 2009 HUDORA GmbH. All Rights Reserved.
"""

from django.core.files.storage import Storage
from django.conf import settings
from django.core.files import File

from cStringIO import StringIO
import couchdb.client

from urlparse import urljoin
from urllib import quote_plus
import os.path

from huTools.couch import setup_couchdb


DEFAULT_SERVER = "http://couchdb.local:5984/"


class CouchDBFile(File):
    """
    CouchDBFile - a Django File-like class for CouchDB documents.
    """

    def __init__(self, name, storage, mode):
        self._name = name
        self._storage = storage
        self._mode = mode
        self._is_dirty = False

        try:
            self._doc = self._storage.get_document(name)

            dummy, ext = os.path.split(name)
            if ext:
                filename = "content." + ext
            else:
                filename = "content"
            attachment = self._storage.db.get_attachment(self._doc, filename=filename)
            if not attachment and filename != "content":
                attachment = self._storage.db.get_attachment(self._doc, filename='content')
            self.file = StringIO(attachment)
        except couchdb.client.ResourceNotFound:
            if 'r' in self._mode:
                raise ValueError("The file cannot be reopened.")
            else:
                self.file = StringIO()
                self._is_dirty = True

    @property
    def size(self):
        return self._doc['size']

    def write(self, content):
        if 'w' not in self._mode:
            raise AttributeError("File was opened for read-only access.")
        self.file = StringIO(content)
        self._is_dirty = True

    def close(self):
        if self._is_dirty:
            self._storage._put_file(self._name, self.file.getvalue())
        self.file.close()


class CouchDBStorage(Storage):
    """
    CouchDBStorage - a Django Storage class for CouchDB.

    The CouchDBStorage can be configured in settings.py, e.g.:
    COUCHDB_STORAGE_OPTIONS = {'server': "http://couchdb.example.com:5984", 'database': 'produktpass_images'}

    Alternatively, the configuration can be passed as a parameter 'option' as a dictionary.
    """

    def __init__(self, **kwargs):
        config = {}
        if hasattr(settings, "COUCHDB_STORAGE_OPTIONS"):
            config.update(settings.COUCHDB_STORAGE_OPTIONS)
        config.update(kwargs)
        self.base_url = config.get('server', DEFAULT_SERVER)
        self.db = setup_couchdb(self.base_url, config.get('database'))

    def _put_file(self, name, content):
        doc = {}
        if name in self.db:
            doc = self.db[name]
        doc['size'] = len(content)
        self.db[name] = doc
        self.db.put_attachment(self.db[name], content, filename='content')
        return name

    def get_document(self, name):
        return self.db.get(name)

    def _open(self, name, mode='rb'):
        couchdb_file = CouchDBFile(name, self, mode=mode)
        return couchdb_file

    def _save(self, name, content):
        content.open()
        if hasattr(content, 'chunks'):
            content_str = ''.join(chunk for chunk in content.chunks())
        else:
            content_str = content.read()
        name = name.replace('/', '-')
        return self._put_file(name, content_str)

    def exists(self, name):
        return name in self.db

    def size(self, name):
        doc = self.get_document(name)
        if doc:
            return doc['size']
        return 0

    def url(self, name):
        return urljoin(self.base_url, os.path.join(quote_plus(self.db.name), quote_plus(name), 'content'))

    def delete(self, name):
        try:
            del self.db[name]
        except couchdb.client.ResourceNotFound:
            pass

    #def listdir(self, name):
    # _all_docs?
    #    pass



"""Test hudjango.auth.decorator functionality."""

from django.test import TestCase
from django.http import HttpResponse
from hudjango.auth.decorators import require_login
from django.test.client import Client
from django.contrib.auth.models import User


def fully_decorated(request):
    """Expected __doc__"""
    return HttpResponse('<html><body>dummy</body></html>')
fully_decorated.anything = "Expected __dict__"

fully_decorated = require_login(fully_decorated)


class DecoratorsTest(TestCase):

    def test_attributes(self):
        """
        Tests that django decorators set certain attributes of the wrapped
        function.
        """
        self.assertEquals(fully_decorated.__name__, 'fully_decorated')
        self.assertEquals(fully_decorated.__doc__, 'Expected __doc__')
        self.assertEquals(fully_decorated.__dict__['anything'], 'Expected __dict__')

    def test_require_login(self):
        """
        Verifies that an unauthenticated user attempting to access a
        login_required view gets redirected to the login page and that
        an authenticated user is let through.
        """
        
        view_url = '/auth_decorators/require_login/'
        client = Client()
        user = User.objects.create_user('testuser', 'nobody@hudora.de', 'testpassword')
        user.save()
        
        # unauthenticated
        response = client.get(view_url)
        self.assertEqual(response.status_code, 401)
        
        # HTTP-authenticated
        auth = 'basic %s' % 'testuser:testpassword'.encode('base64')
        response = client.get(view_url, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'access!')
        
        # Django/Session-authenticated
        client = Client()
        client.login(username='testuser', password='testpassword')
        response = client.get(view_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'access!')
        
        # mismatch between session and HTTP-authentication, invalid http auth
        client = Client()
        client.login(username='testuser', password='testpassword')
        auth = 'basic %s' % 'nouser:nopassword'.encode('base64')
        response = client.get(view_url, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 401)
        self.assertNotContains(response, 'access!', status_code=401)
        
        # invalid HTTP-authentication
        client = Client()
        client.login(username='testuser', password='testpassword')
        auth = 'basic %s' % 'nouser:nopassword'.encode('base64')
        response = client.get(view_url, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 401)
        self.assertNotContains(response, 'access!', status_code=401)



# -*- coding: utf-8 -*-

"""Validators extending DJANGO.

See http://www.djangoproject.com/documentation/forms/#validators for more documentation on Django's
validators. Hopfully the validators below are all self explanatory."""

# Coded 2006 by Maximillian Dornseif for HUDORA GmbH
# You may use this under a "BSD License".

try:
    #1.0
    from django.core.exceptions import ValidationError
except ImportError:
    #pre r8616
    from django.core.validators import ValidationError


class IsLessOrEqualThanOtherField(object):
    """Takes a field name and validates that the current field being validated has a value that is less
    than (or equal to) the other field's value. Comparisons are done using floats.

    This is a Variation of Django's IsLessThanOtherField."""

    def __init__(self, other_field_name, error_message = None):
        self.always_test = True
        self.other, self.error_message = other_field_name, error_message
        if not self.error_message:
            self.error_message = 'Dieses Feld darf nicht größer als Feld %r sein.' % self.other

    def __call__(self, field_data, all_data):
        if not field_data:
            return
        try:
            field_data = float(field_data)
        except Exception:
            try:
                field_data = int(field_data)
            except Exception:
                return
        try:
            other_data = float(all_data[self.other])
        except Exception:
            try:
                other_data = int(all_data[self.other])
            except Exception:
                return

        if field_data > other_data:
            raise ValidationError(self.error_message)


class IsLessOrEqualThan(object):
    """Validates that the current field is lower than the given value."""

    def __init__(self, maxvalue, error_message = None):
        self.always_test = True
        self.maxvalue, self.error_message = float(maxvalue), error_message
        if not self.error_message:
            self.error_message = 'Dieses Feld darf nicht größer als %r sein.' % self.maxvalue

    def __call__(self, field_data, dummy):
        if not field_data:
            return
        try:
            field_data = float(field_data)
        except Exception:
            try:
                field_data = int(field_data)
            except Exception:
                return

        if field_data > self.maxvalue:
            raise ValidationError(self.error_message)



#!/usr/bin/env python
# encoding: utf-8
"""
regressiontests/serializers/models.py

Created by Maximillian Dornseif on 2009-08-17.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

from django.db import models

class Place(models.Model):
    name = models.CharField(max_length=50)
    address = models.CharField(max_length=80)
    last_visit = models.DateTimeField()
    
    def __unicode__(self):
        return u"%s the place" % self.name
    
    def get_absolute_url(self):
        return 'http://testserver/place/%d/' % self.id



#!/usr/bin/env python
# encoding: utf-8
"""
dedupingS3storage.py

Created by Maximillian Dornseif on 2010-05-04.
Copyright (c) 2010 HUDORA. All rights reserved.
"""

from boto.s3.key import Key
from storages.backends import s3boto
import hashlib
import mimetypes
import os

def _compute_hash(fp):
    BufferSize = 4096*100
    m = hashlib.md5()
    fp.seek(0)
    s = fp.read(BufferSize)
    while s:
        m.update(s)
        s = fp.read(BufferSize)
    fp.seek(0)
    return m.digest().encode('base64').replace('/', 'H').replace('+', 'D').strip('\n=')
    

class DedupingS3storage(s3boto.S3BotoStorage):
    """Based on S3BotoStorage this ensures that similar files are only saved once.
    
    Does NOT support deletion of files."""
    
    def _save(self, name, content):
        name = self._clean_name(name)
        headers = self.headers
        content_type = mimetypes.guess_type(name)[0] or "application/x-octet-stream"
        
        if self.gzip and content_type in self.gzip_content_types:
            content = self._compress_content(content)
            headers.update({'Content-Encoding': 'gzip'})
        
        headers.update({
            'Content-Type': content_type,
            'Content-Length' : len(content),
        })
        
        newname = _compute_hash(content)
        extension =  mimetypes.guess_extension(content_type)
        if extension:
            newname = newname + extension
        content.name = newname
        k = self.bucket.get_key(newname)
        if not k:
            k = self.bucket.new_key(newname)
            k.set_metadata('original_filename', name)
        k.set_contents_from_file(content, headers=headers, policy=self.acl)
        return newname

    def delete(self, name):
        pass

    def url(self, name):
        s3url = super(DedupingS3storage, self).url(name)
        if self.acl == 'public-read':
            s3url = s3url.split('?')[0]
        return s3url



#!/usr/bin/env python
# encoding: utf-8
"""
hudjango/serializers.py - serialize QuerySets or lit of objects to Excel/XLS.

http://docs.djangoproject.com/en/dev/topics/serialization/ is inspiration for this.

Created by Maximillian Dornseif on 2009-08-17.
Copyright (c) 2009 HUDORA. All rights reserved.
"""


import django.db.models.fields.files
import django.contrib.sites.models
import datetime
import types
import xlwt
from cStringIO import StringIO

xlwt.UnicodeUtils.DEFAULT_ENCODING = 'utf-8'


def _write_row(worksheet, row, pos_row):
    """Write a row to the current worksheet."""
    col = 0
    for cell in row:
        if isinstance(cell, list):
            worksheet.write(pos_row, col, ', '.join(cell))
        elif isinstance(cell, datetime.datetime):
            worksheet.write(pos_row, col, cell.strftime('%Y-%m-%d %H:%M'))
        # elif isinstance(cell, django.db.models.fields.files.ImageFieldFile):
        #     cellcontent = ''
        #     if getattr(cell, 'name', None):
        #         cellcontent = xlwt.Formula('HYPERLINK("%s";"%s")'
        #                                    % (huimages.imageurl(getattr(cell, 'name')), "Bild"))
        #         worksheet.write(pos_row, col, cellcontent)
        else:
            worksheet.write(pos_row, col, cell)
        col += 1
    return pos_row + 1


def queryset_to_xls(queryset, fields=None, headings=None):
    """Takes a Django queryset or a list of Objects and returns an XLS Sheet.

    Returns '' if the Queryset is empty."""

    if not queryset:
        return ''
    if not headings:
        headings = {}

    if not fields and hasattr(queryset, 'model'):
        fields = sorted([field.name for field in queryset.model._meta.fields])
    if not fields and queryset and hasattr(queryset[0], '_meta'):
        fields = sorted([field.name for field in queryset[0]._meta.fields])
    assert fields, "must be called with a list of fieldnames or a Queryset"

    workbook = xlwt.Workbook()

    # 1. Worksheet, ausführlich
    worksheet = workbook.add_sheet(u'Details')
    pos_row = 0

    # write headings
    headingrow = ['']
    for name in fields:
        headingrow.append(headings.get(name, name))
    pos_row = _write_row(worksheet, headingrow, pos_row)

    for obj in list(queryset):
        absurl = obj.get_absolute_url()
        if not absurl.startswith('http://'):
            absurl = 'http://%s%s' % (django.contrib.sites.models.Site.objects.get_current().domain, absurl)
        linktext = unicode(obj).replace('"', '') # Extremly curude but get's the job done
        row = [xlwt.Formula('HYPERLINK("%s";"%s")' % (absurl, linktext))]
        for name in fields:
            if hasattr(getattr(obj, name), '__call__'):
                # call 'name' is possible ...
                row.append(getattr(obj, name)())
            else:
                # if not, use value
                row.append(getattr(obj, name))
        pos_row = _write_row(worksheet, row, pos_row)
    pos_row = _write_row(worksheet, [u'Stand:', unicode(datetime.datetime.now())], pos_row)

    buf = StringIO()
    workbook.save(buf)
    return buf.getvalue()



# encoding: utf-8

import couchdb
from hudjango.management.commands.couchdbcommand import CouchDBCommand
from django.core.management.base import CommandError


class Command(CouchDBCommand):
    help = """Drop a couchdb database"""

    def handle(self, *args, **options):
        if len(args) != 1:
            raise CommandError("You need to specify exactly one argument as database name")
        database = args[0]
        server = couchdb.client.Server(options['server'])
        try:
            server.delete(database)
        except couchdb.client.ResourceNotFound:
            pass

        print "Database '%s' dropped succesfully" % database



"""This module collects several extensions to Django we found useful at HUDORA.
See http://www.djangoproject.com/ and https://cybernetics.hudora.biz/projects/wiki/huDjango

"""

import datetime
from django.contrib.admin.models import LogEntry, ADDITION, CHANGE, DELETION
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import smart_unicode


class PrinterChooser(object):
    """PrinterChooser allows the user in a HTML form to choose a printer. It remembers the choice.
    PrinterChooser() must be initialized with

    * A Django Request Object
    * A list of possible printer names. The first name is the default choice.
    * An optional identifier if you want to use PrinterChooser() for different kinds of printers.
      We use 'a4', 'a4color' and 'label'.

    To use it your view must be structured like this:

        def myview(request):
            printer = PrinterChooser(request, ('DruckerErikBloodaxe', 'DruckerNapster'), 'label')
            ...
            print "Currently selected printer is %s" % printer.name
            response = render_to_response('huLOG/packstuecke_bearbeiten.html',
                                          printer.update_context({'lieferung': lieferung}),
                                          context_instance=RequestContext(request))
            populate_xheaders(request, response, Sendung, lieferung.id)
            return printer.update_response(response)


    In your template do:

        {% load hudjango %}
        ...
        <form ...>
        {% printer_choice %}
        ...
        </form>

    See http://static.23.nu/md/Pictures/ZZ5FB43EF0.png for an example.
    """

    def __init__(self, request, choices, name='default'):
        self.request = request
        self.choices = choices
        self.classname = name

    def update_response(self, response):
        """Updates an django Response Object to set the printer choice.

        Usually called in the last line of a Django view function:
            return printer.update_response(render_to_response(...))
        """

        printer_choice = self.request.POST.get('printer_choice', None)
        if printer_choice:
            max_age = 365*24*60*60  # one year
            expires = datetime.datetime.strftime(datetime.datetime.utcnow()
                        + datetime.timedelta(seconds=max_age), "%a, %d-%b-%Y %H:%M:%S GMT")
            response.set_cookie('printer_choice_%s' % self.classname, printer_choice,
            max_age=max_age, expires=expires)
            self.request.session['printer_choice'] = printer_choice
        return response

    def update_context(self, contextdict=None):
        """Update a Django template context (dict) in regard to the printer choices.

        Usually called in a Django view function as a parameter to render_to_response():
            response = render_to_response('foo.html', printer.update_context({'var1': 'val1', ...}))
        """

        if not contextdict:
            contextdict = {}
        contextdict.update({'printer_choice_list': self.choices,
                            'printer_choice': self.name})
        return contextdict

    @property
    def name(self):
        """Represents the currently choosen printer name."""

        printer_choice = self.request.POST.get('printer_choice')
        if not printer_choice:
            printer_choice = self.request.COOKIES.get('printer_choice_%s' % self.classname)
        if not printer_choice:
            self.request.session.get('printer_choice', self.choices[0])
        return printer_choice


def log_action(obj, action, user=None, message='', reprstr=None):
    """Adds a Django Admin Log entry.

    Arguments
     * obj: The object which is influenced by this action
     * action: one of the actions in django.contrib.admin.models; ADDITION, CHANGE, DELETION
               or the string representation
     * user: the user which is responsible for this action. Defaults to user 'logger'
     * message: A short message containing information about the action
     * reprstr: a representation of the object
    """
    # from https://svn.python.org/conference/django/trunk/pycon/propmgr/changelog.py

    actions = {'ADDITION': ADDITION, 'CHANGE': CHANGE, 'DELETION': DELETION}
    if isinstance(action, basestring):
        action = actions[action]

    content_type = ContentType.objects.get_for_model(type(obj))
    if user is None:
        try:
            log_user = User.objects.get(username='logger')
            uid = log_user.id
        except User.DoesNotExist:
            uid = 1
    else:
        uid = user.id

    if reprstr is None:
        reprstr = smart_unicode(repr(obj))
    else:
        reprstr = smart_unicode(reprstr)
    LogEntry.objects.log_action(uid, content_type.id, obj.id, reprstr, action, message)



import unittest
from hudjango.middleware import threadlocals
from random import random
from thread import start_new_thread as launch


class TestThreadLocals(unittest.TestCase):

    def test_LocalImplementation(self):
        lo = threadlocals.local()
        lo.foo = "bar"
        self.assertEqual(lo.foo, "bar")

        def fetchfoo():
            return lo.foo

        self.assertEqual(fetchfoo(), "bar")
        launch(self.assertRaises, (AttributeError, fetchfoo))

        def getsetlocal():
            lo.foo = 'bat'
            self.assertEqual(lo.foo, "bat")

        launch(getsetlocal, ())
        self.assertEqual(lo.foo, "bar")

    def test_GetSetLocalUser(self):
        self.assert_(threadlocals.get_current_user() is None)
        val = random()
        threadlocals.set_current_user(val)
        self.assertEquals(val, threadlocals.get_current_user())
    
    def test_GetSetRemoteIP(self):
        self.assert_(threadlocals.get_remote_ip() is None)
        val = random()
        threadlocals.set_remote_ip(val)
        self.assertEquals(val, threadlocals.get_remote_ip())
        
    def test_ThreadLocalsClass(self):
        request = threadlocals.ThreadLocals() #almost any class except object() would work
        request.user = user = random()
        request.META = {}
        request.META['REMOTE_ADDR'] = addr = random()
        threadlocals.ThreadLocals().process_request(request)
        self.assertEqual(user, threadlocals.get_current_user())
        self.assertEqual(addr, threadlocals.get_remote_ip())

suite = unittest.TestLoader().loadTestsFromTestCase(TestThreadLocals)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
backends.py - extended authentication login for django

Currently this module provides two authentication backends.

EmailBackend    - allows login using the E-Mail adress instead of the password.
ZimbraBackend   - allows authentication against a Zimbra-LDAP server with E-Mail address and Zimbra password.

Usage:

If you have installed huDjango you can edit the settings.py of your Django
Project to use the new authentication Backend. Add

AUTHENTICATION_BACKENDS = (
    'hudjango.auth.backends.EmailBackend', 
    'django.contrib.auth.backends.ModelBackend',
)

Created by Maximillian Dornseif on 2008-02-28.
Copyright (c) 2008 HUDORA. All rights reserved.
"""

import ldap
from django.conf import settings 
from django.contrib.auth.models import User
import django.contrib.auth.backends


class EmailBackend(django.contrib.auth.backends.ModelBackend):
    """
    Authenticate against django.contrib.auth.models.User the same way Django
    does by default except that the users can use their E-Mail
    address instead of the unsername to authenticate. This obviously
    leads to an awful lot of trouble if the E-Mail adresses in your 
    User Database are not unique.
    """

    def authenticate(self, username=None, password=None):
        """Authenticate a user with password."""
        
        # our auth strategy
        try:
            # 1. see if we find a user object with that username
            user = User.objects.get(username=username)
            if user.check_password(password):
                return user
        except User.DoesNotExist:
            # 2. see if we find a user object with that email adress
            users = User.objects.filter(email=username)
            if len(users) == 1 and users[0].check_password(password):
                return users[0]
        return None
    

class ZimbraBackend(django.contrib.auth.backends.ModelBackend):
    """
    Authenticate against a Zimbra Server via LDAP.
    
    Users have to use their full E-Mail adress and Zimbra Passwort to log in.
    This obviously leads to an awful lot of trouble if the E-Mail adresses in your 
    User Database are not unique. It also gets in trouble if you have adresses which
    are only destinguished by a dot. So 'mdornseif' 'm.dornseif' and 'mdo.rnseif' can't
    coexist.
    
    The Backend creates a new entry in the Django User Database if authentication
    against Zimbra is successfull but no matching E-Mail in Django adress is found.
    the new user gets the attribute is_staff = True.
    
    You can set your LDAP Servername in settings using the variable LDAP_SERVER_NAME. E.g.:
    
    LDAP_SERVER_NAME = 'ldap.hudora.biz'
    
    The code assumes the LDAP server allows "annonymous bind" (which is the default with Zimbra).
    
    This code is based in Django Ticket 2507 - http://code.djangoproject.com/attachment/ticket/2507
    and code from Christian N Klein.
    """
    
    def authenticate(self, username=None, password=None):
        """Authenticate a user with password.
        
        Might also create the user in the Django User DB.
        """
        
        servername = 'mail.hudora.biz'
        if hasattr(settings, 'LDAP_SERVER_NAME'):
            servername = settings.LDAP_SERVER_NAME
        lconn = ldap.open(servername)
        lconn.protocol_version = ldap.VERSION3
        result_id = lconn.search('', ldap.SCOPE_SUBTREE, 'mail=%s' % (username, ),
                             ['uid', 'displayName', 'givenname', 'sn', 'mail', 'userPassword'])
        dummy, result_data = lconn.result(result_id, 0)
        
        # If the user does not exist in LDAP, Fail.
        if (len(result_data) != 1):
            lconn.unbind_s()
            return None
        uid, result_dict = result_data[0]
        
        # Attempt to bind to the user's DN (verify password)
        try:
            lconn.simple_bind_s(uid, password)
        except ldap.INVALID_CREDENTIALS: # Failed user/pass 
            # password wrong, exit
            return None 
        finally:
            lconn.unbind_s() 
        
        # get user DB from Django
        users = User.objects.filter(email=username)
        # in case we get more than user back, we want a stable sort order.
        users = sorted(users, cmp=lambda a, b: len(a.username)<len(b.username))
        if len(users) == 0:
            # generate user from LDAP in Django
            user = User()
            user.username = result_dict.get('uid')[0].replace('.', '')
            user.first_name = result_dict.get('givenName')[0]
            user.last_name = result_dict.get('sn')[0]
            user.email = result_dict.get('mail')[0]
            user.is_staff = True
            user.is_active = True
            user.set_password(password)
            user.save()
        else:
            user = users[0]
            user.set_password(password)
            user.save()
        return user



#!/usr/bin/env python
# encoding: utf-8

"""
defaulting.py

Django fields with more elaborate default values.

Created by Maximillian Dornseif on 2006-11-12. Consider it BSD licensed.
"""

from django.db import models
from django.db.models import signals


class DefaultingCharField(models.CharField):
    """Results in a dynamic default value - basically if no value is given the field is filled from an 
       other field during saving.
    """
    # we might  be able to get the same effect with a callable default but I dont know how to ensure 
    # that the field  referenced is already set when the callable is executed.
    def __init__(self, default_from_field=None, *args, **kwargs):
        self.default_from_field = default_from_field
        super(DefaultingCharField, self).__init__(*args, **kwargs)
    
    def _set_default(self, instance=None, **kwargs):
        """Will be called by pre_save to set a default value if the attribute is not set."""
        if not self.default_from_field:
            return
        if not getattr(instance, self.attname):
            if len(getattr(instance, self.default_from_field)) <= self.max_length:
                setattr(instance, self.attname, instance.name)
    
    def contribute_to_class(self, cls, name):
        """Adds a pre_save signal handler to ensure we can set the defaults before beeing saved."""
        super(DefaultingCharField, self).contribute_to_class(cls, name)
        signals.pre_save.connect(self._set_default, sender=cls)
    
    def get_internal_type(self):
        return 'CharField'
    
#class untitled:
#    def __init__(self):
#        pass

#class untitledTests(unittest.TestCase):
#    def setUp(self):
#        pass

#if __name__ == '__main__':
#    unittest.main()



#!/usr/bin/env python

from django.core.management.base import BaseCommand
from django.contrib.sessions.models import Session
from django.contrib.auth.models import User
from django.utils.termcolors import colorize
import sys

class Command(BaseCommand):
    
    def handle(self, session_id, *args, **options):
        """Extract user id from session and display username."""

        try:
            session = Session.objects.get(pk=session_id)
        except Session.DoesNotExist:
            sys.stderr.write("Hey, I know who it is, it's %s!\n" % colorize('YOU', fg='red'))
            return
        try:
            user_id = session.get_decoded()['_auth_user_id']
            user = User.objects.get(pk=user_id)
            sys.stderr.write("Session is bound to user %s (%s)\n" % (colorize(user.get_full_name(), fg='green'), colorize(user.email, fg='green')))
        except KeyError:
            sys.stderr.write('No user id in session data\n')
        except User.DoesNotExist:
            sys.stderr.write('User from session data does not exist\n')



"""
Wrapper for loading templates from a CouchDB.
"""

from django.conf import settings
from django.template import TemplateDoesNotExist
from django.utils._os import safe_join

from urlparse import urljoin
import logging


def _setup_couchdb(servername, database):
    """Get a connection handler to the CouchDB Database, creating it when needed."""
    
    server = couchdb.client.Server(servername)
    if database in server:
        return server[database]
    else:
        return server.create(database)


try:
    from huTools.couch import setup_couchdb
except ImportError:
    import couchdb.client
    import warnings
    warnings.warn("Your huTools installation does not contain the couch module.", warnings.ImportWarning)
    setup_couchdb = _setup_couchdb


def load_template_source(template_name, template_dirs=None):
    
    if not hasattr(settings, 'TEMPLATE_COUCHDB_SERVER'):
        error_msg = "CouchDB Template Loader not configured"
        raise TemplateDoesNotExist, error_msg
    
    server = settings.TEMPLATE_COUCHDB_SERVER
    databasename = getattr(settings, 'TEMPLATE_COUCHDB_DATABASE', 'templates')
    database = setup_couchdb(server, databasename)
    
    if template_name in database:
        doc = database[template_name]
        try:
            template = doc['source']
            return template, urljoin(server, databasename, template_name)
        except KeyError:
            error_msg = "Document does not contain template code"
    else:
        error_msg = "No template '%s' found" % template_name
    raise TemplateDoesNotExist, error_msg
load_template_source.is_usable = True



#!/usr/bin/env python
# encoding: utf-8
"""
tests/auth_decorators/views.py

Created by Maximillian Dornseif on 2009-08-15.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

from django.http import HttpResponse
from hudjango.auth.decorators import require_login


@require_login
def require_login_view(request):
    return HttpResponse('access!', status=200, mimetype='text/plain')



#!/usr/bin/env python
# encoding: utf-8

"""
couchdbfield.py

Django field for storing information about a CouchDB document

Created by Christian Klein on 2010-02-24.
(c) 2010 HUDORA GmbH. All rights reserved.
"""

from django.db import models

import couchdb.client
from uuid import uuid4
from huTools.couch import setup_couchdb

from urlparse import urljoin


####################################################
# put somewhere else...
from django import forms
from django.utils.safestring import mark_safe

class CouchDBWidget(forms.widgets.Widget):
    def render(self, name, value, attrs=None):
        return mark_safe(u'<input size="64" name="%s" value="%s" />' % (name, value))

class CouchDBFormField(forms.Field):
    #widget = CouchDBWidget
    def __init__(self, *args, **kwargs):
        super(CouchDBFormField, self).__init__(*args, **kwargs)
        self.widget = CouchDBWidget()
####################################################


class CouchDBDocument(couchdb.client.Document):
    """CouchDBDocument"""
    
    def __unicode__(self):
        return urljoin(self.server, '%s/%s' % (self.database, self.id))
    
    def set_location(self, server, database):
        self.server = server
        self.database = database
    
    def get_location(self):
        return self.server, self.database, self.id
    
    def save(self):
        """
        Save document to CouchDB.
        
        If the document exists already, get previous revision and update document
        """
        
        if not (hasattr(self, 'server') or hasattr(self, 'database')):
            raise RuntimeError("object has not been told where to save!")
        
        couchdb =  setup_couchdb(self.server, self.database)
        if self.id in couchdb:
            doc = couchdb[self.id]
            self.update(doc)
        couchdb[self.id] = self


class CouchDBField(models.CharField):

    description = "Store id of a CouchDB document"
    
    __metaclass__ = models.SubfieldBase
    
    def __init__(self, server=None, database=None, **kwargs):
        self.server, self.database = server, database
        self._couchcon = None
        
        # Keep track of CouchDB document id to prevent creation of new documents
        self.doc_id = None
        kwargs['max_length'] = 256
        
        super(CouchDBField, self).__init__(**kwargs)
    
    def _connection(self, server=None, database=None):
        """Cache connection to CouchDB"""
        if self._couchcon is None:
            self._couchcon = setup_couchdb(server, database)
        return self._couchcon
    
    def to_python(self, value):
        """Convert to Python object"""
        
        if value == '' or value is None:
            doc = CouchDBDocument()
        elif isinstance(value, dict):
            doc = CouchDBDocument(_id=self.doc_id, **value)
        else:
            try:
                server, database, doc_id = value.split('#')
                doc = CouchDBDocument(self._connection(server, database)[doc_id])
            except:
                print "ui!"
                doc = CouchDBDocument()
        doc.set_location(self.server, self.database)
        return doc
    
    def get_db_prep_value(self, value):
        """Convert Python object to database value"""
        return "#".join(value.get_location())
    
    def get_db_prep_save(self, value):
        """
        Save document to CouchDB when saving Django object.
        
        If it's a new document, generate an id.
        """
        
        if (not '_id' in value) or (value.get('_id') is None):
            value['_id'] = uuid4().hex
            self.doc_id = value['_id']
        value.save()
        return self.get_db_prep_value(value)
    
    def contribute_to_class(self, cls, name):
        super(CouchDBField, self).contribute_to_class(cls, name)
        models.signals.post_delete.connect(self.delete, sender=cls)
    
    def delete(self, **kwargs):
        """Delete the document from the CouchDB database."""
        if self.doc_id:
            doc = self._connection(self.server, self.database)[self.doc_id]
            self._connection(self.server, self.database).delete(doc)
    
    def formfield(self, form_class=CouchDBFormField, **kwargs):
        defaults = {'required': not self.blank, 'label': self.verbose_name, 'help_text': self.help_text}
        defaults.update(kwargs)
        return form_class(**defaults)



#!/usr/bin/env python
# encoding: utf-8
"""
middleware.py tracks clients by setting a persistent unique coockie.

You can access it via request.clienttrack_uid in your views.
There is also request.clienttrack_first_visit and request.clienttrack_last_visit
available to your views. See ClientTrackMiddleware for details.

Always keep in mind that Coockies can easily be faked.

This has no dependencies on django.contrib.auth and django.contrib.sessions.

Created by Maximillian Dornseif on 2009-02-07.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

import base64
import hashlib
import random
import time
from django.utils.http import cookie_date


def _gen_uid():
    return base64.b32encode(hashlib.md5("%f-%f" % (random.random(), time.time())).digest()).rstrip('=')
    

class ClientTrackMiddleware(object):
    """The ClientTrackMiddleware tracks Clients (Browsers) by setting a unique cookie.
    
    Assuming cookies work as advertised you can access the following variables in your view functions:
    
    * request.clienttrack_uid (always set) - unique ID we assigned to the client.
      Keep in mind that this is trival to fake.
    * request.clienttrack_first_visit - time.time() value at the very first time the client accessed our
      server or None for first time visitors.
    * request.clienttrack_last_visit - time.time() value at the very time the client last accessed our
      server or None.
    """
    
    def process_request(self, request):
        if '_hda' in request.COOKIES:
            parts = request.COOKIES.get('_hda').split(',')
            request.clienttrack_first_visit, request.clienttrack_uid \
                = parts[:2]
            if len(parts) > 3:
                request.clienttrack_last_visit = parts[2]
            else:
                request.clienttrack_last_visit = None
        else:
            request.clienttrack_uid = _gen_uid()
            request.clienttrack_last_visit = request.clienttrack_first_visit = None
    
    def process_response(self, request, response):
        # warning: when process_response() is called it might be, that process_request()
        # was NOT called before on the process object
        if (not getattr(request, 'clienttrack_prohibit', False)) or \
          not getattr(request, 'clienttrack_first_visit', None):
            # even if clienttrack_prohibit is True, we we set the cookie for first time visitors. 
            if not getattr(request, 'clienttrack_first_visit', None):
                request.clienttrack_first_visit = time.time()
            max_age = 3*365*24*60*60  # 3 years
            expires_time = time.time() + max_age
            expires = cookie_date(expires_time)
            response.set_cookie('_hda', "%s,%s,%f" % (request.clienttrack_first_visit,
                                                      getattr(request, 'clienttrack_uid', _gen_uid()),
                                                      time.time()),
                                max_age=max_age, expires=expires)
        return response



# encoding: utf-8

"""Image field which scales images on demand.

This acts like Django's ImageField but in addition can scale images on demand. Scaled Images are put in
<settings.MEDIA_ROOT>/,/<originalpath>. The avialable scaling factors are hardcoded in the dictionary _sizes.
If the dimensions there are followed by an '!' this means the images should be cropped to exactly this size.
Without this the images are scaled to fit in the given size without changing the aspect ratio of the image.

Scaled versions of the images are generated on the fly using PIL and then kept arround in the Filesystem. 

Given a model like

class Image(models.Model):
    path       = ScaledImageField(verbose_name='Datei', upload_to='-/product/image')
    [...]

you can do  the following:

>>> img.path  
'-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg'
>>>img.get_path_url()
'/media/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg'
>>> img.get_path_size()
417119L
>>> img.get_path_width()
1584
>>> img.get_path_height()
2889

All well known metods from ImageField are supported. The new functionality is available via img.path_scaled - this returns an
Imagescaler instance beeing able to play some nifty tricks:

>>> img.path_scaled().svga()
'/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg/svga.jpeg'
>>> img.path_scaled().svga_path()
'/usr/local/web/media/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg/svga.jpeg'
>>> img.path_scaled().svga_dimensions()
(328, 600)
>>> img.path_scaled().svga_tag()
'<img src="/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg/svga.jpeg" width="328" height="600" />'
>>> img.path_scaled().thumb_dimensions()
(50, 91)
>>> img.path_scaled().square_dimensions()
(75, 76)

Created August 2006 by Maximillian Dornseif. Updated in early 2009.
Consider it BSD licensed.
"""

import Image 
import os
import urlparse
from django.conf import settings
from django.db.models import ImageField
from django.utils.functional import curry
from django.utils.html import escape 
from django.utils.safestring import mark_safe 

_sizes = {'mini': "23x40",
          'thumb': "50x200", 
          'sidebar': "179x600",
          'small': "240x160",
          'medium': "480x320", 
          'full': "477x800",
          'svga': "800x600", 
          'xvga': "1024x768",
          'square': "75x75!"} 


def _scaleImage(width, height, image):
    """
    This function will scale an image to a given bounding box. Image
    aspect ratios will be conserved and so there might be blank space
    at two sides of the image if the ratio isn't identical to that of
    the bounding box.
    """
    #from http://simon.bofh.ms/cgi-bin/trac-django-projects.cgi/file/stuff/branches/magic-removal/image.py
    lfactor = 1
    width, height = int(width), int(height)
    (xsize, ysize) = image.size
    if xsize > width and ysize > height:
        lfactorx = float(width) / float(xsize)
        lfactory = float(height) / float(ysize)
        lfactor = min(lfactorx, lfactory)
    elif xsize > width:
        lfactor = float(width) / float(xsize)
    elif ysize > height:
        lfactor = float(height) / float(ysize)
    res = image.resize((int(float(xsize) * lfactor), int(float(ysize) * lfactor)), Image.ANTIALIAS)
    return res


def _cropImage(width, height, image):
    """
    This will crop the largest block out of the middle of an image
    that has the same aspect ratio as the given bounding box. No
    blank space will be in the thumbnail, but the image isn't fully
    visible due to croping.
    """
    #from http://simon.bofh.ms/cgi-bin/trac-django-projects.cgi/file/stuff/branches/magic-removal/image.py
    # moderately modified
    width, height = int(width), int(height)
    lfactor = 1
    (xsize, ysize) = image.size
    if xsize > width and ysize > height:
        lfactorx = float(width) / float(xsize)
        lfactory = float(height) / float(ysize)
        lfactor = max(lfactorx, lfactory)
    newx = int(float(xsize) * lfactor)
    newy = int(float(ysize) * lfactor)
    res = image.resize((newx, newy), Image.ANTIALIAS)
    leftx = 0
    lefty = 0
    rightx = newx
    righty = newy
    if newx > width:
        leftx += (newx - width) / 2
        rightx -= (newx - width) / 2
    elif newy > height:
        lefty += (newy - height) / 2
        righty -= (newy - height) / 2
    res = res.crop((leftx, lefty, rightx, righty))
    return res


class Imagescaler(object):
    """Class whose instances scale an image on the fly to desired properties.
    
    For each set of dimensions defined in _sizes imagescaler has a set of functions, e.g. for 'small':
    
    o.small() = return the URL of the small version of the image
    o.small_path() - return the absolute  pathe in the filesystem for the  image
    o.small_dimensions() - return (width,  heigth)
    o.small_tag() - return a complete image tag for use in XHTML
    
    >>> img.path_scaled().svga()
    '/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg_svga.jpeg'
    >>> img.path_scaled().svga_path()
    '/usr/local/web/media/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg_svga.jpeg'
    >>> img.path_scaled().svga_dimensions()
    (328, 600)
    >>> img.path_scaled().svga_tag()
    '<img src="/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg_svga.jpeg" width="328" height="600" />'
    """

    def __init__(self, field, obj):
        self.field = field
        self.parent_obj = obj
        self.original_image = getattr(self.parent_obj, self.field.attname)
        self.original_image_path = os.path.join(settings.MEDIA_ROOT, str(self.original_image))
        self.mangled_name = str(self.original_image) # md5.new('sehkr1tt-%s-%r-%r' % (str(self.original_image), time.time(), id(self))).hexdigest()
        self.scaled_image_base = os.path.join(settings.MEDIA_ROOT, ',', self.mangled_name)
        self.broken_image = None
        # if broken.gif exists we send that if there are any problems during scaling
        if not os.path.exists(self.original_image_path):
            self.broken_image = os.path.join(settings.MEDIA_ROOT, 'broken.gif') 
        for size in _sizes:
            setattr(self, '%s_path' % (size), curry(self.scaled_filename, size))
            setattr(self, '%s' % (size), curry(self.scaled_url, size))
            setattr(self, '%s_dimensions' % (size), curry(self.scaled_dimensions, size))
            setattr(self, '%s_tag' % (size), curry(self.scaled_tag, size))

    def scaled_filename(self, size='thumb'):
        """Scales an image according to 'size' and returns the filename of the scaled image."""
        if self.broken_image:
            return self.broken_image
        outpath = "%s_%s.jpeg" % (self.scaled_image_base, size)
        if not os.path.exists(outpath):
            if size not in _sizes:
                width, height = size.split('x')
            else:
                width, height = _sizes[size].split('x')
            img = Image.open(self.original_image_path)
            if height.endswith('!'):
                height = height.strip('!')
                img = _cropImage(width, height, img)
            else:
                img = _scaleImage(width, height, img)
            if img.mode != "RGB":
                img = img.convert("RGB")
            img.save(outpath, "JPEG")
        return outpath

    def scaled_url(self, size='thumb'):
        """Scales an image according to 'size' and returns the URL of the scaled image."""
        if not self.original_image:
            return ''
        outpath = self.scaled_filename(size)
        url = outpath[len(settings.MEDIA_ROOT):]
        if url.startswith('/'):
            url = url[1:]
        return urlparse.urljoin(settings.MEDIA_URL, url).replace('\\', '/')
    
    def scaled_dimensions(self, size='thumb'):
        """Scales an image according to 'size' and returns the dimensions."""
        if not self.original_image:
            return None
        if size.endswith('!'):
            return [int(i) for i in _sizes[size].split('x')]
        return Image.open(self.scaled_filename(size)).size
    
    def scaled_tag(self, size='thumb', *args, **kwargs):
        """Scales an image according to 'size' and returns an XHTML tag for that image.
        
        Additional keyword arguments are added as attributes to  the <img> tag.
        
        >>> img.path_scaled().svga_tag(alt='neu')
        '<img src="/,/-/product/image/0ead6f.jpg/svga.jpeg" width="328" height="600" alt="neu"/>'
        """
        if not self.original_image:
            return ''
        ret = ['<img src="%s"' % escape(self.scaled_url(size))]
        try:
            ret.append('width="%d" height="%d"' % self.scaled_dimensions(size))
        except:
            ret.append('width="50" height="50" alt="Fehler bei der Groessenberechnung" ')
        ret.extend(args)
        for key, val in kwargs.items():
            ret.append('%s="%s"' % (escape(key), escape(val)))
        ret.append('/>')
        return mark_safe(' '.join(ret))


class ScalingImageField(ImageField):
    """This acts like Django's ImageField but in addition can scale images on demand by providing an
    ImageScaler object.
    
    >>> img.path_scaled().svga()
    '/,/-/product/image/0e99d6be8ec0259df920c2d273d1ad6f.jpg/svga.jpeg'
    """

    def __init__(self, verbose_name=None, name=None, width_field=None, height_field=None, auto_rename=True,
                 **kwargs):
        """Inits the ScalingImageField."""
        super(ScalingImageField, self).__init__(verbose_name, name, width_field, height_field, **kwargs)

    def contribute_to_class(self, cls, name):
        """Adds field-related functions to the model."""
        super(ScalingImageField, self).contribute_to_class(cls, name)
        setattr(cls, '%s_scaled' % self.name, curry(Imagescaler, self))

    def get_internal_type(self):
        return 'FileField'



"""
This is a Custom Storage System for Django with CouchDB backend.
Created by Christian Klein.
(c) Copyright 2009 HUDORA GmbH. All Rights Reserved.
"""

from hudjango.storage.CouchDBStorage import CouchDBStorage
huimages = None  # lazy import, see ImageServerStorage.__init__()


class ImageServerStorage(CouchDBStorage):
    """ImageServerStorage - a Django Storage class for the huDjango ImageServer."""
    
    def __init__(self, **kwargs):
        global huimages
        if huimages is None:
            import huimages
        self.typ = kwargs.get('type', None)
        
    def _put_file(self, name, content):
        """Put file on the server."""
        if self.typ:
            return huimages.save_image(content, filename=name, typ=self.typ)
        return huimages.save_image(content, filename=name)
    
    def exists(self, name):
        """Check if an File/Image exists."""
        try:
            huimages.get_size(name)
            return True
        except:
            return False
    
    def size(self, name):
        """Return the dimensions (x, y) of an image."""
        return huimages.get_size(name)
    
    def url(self, name):
        """Return the URL where an Image can be accessed."""
        return huimages.imageurl(name)
    
    def delete(self, name):
        """Delete an Image - not implemented so far."""
        pass
        
    def listdir(self, path):
        """List a directory - not supported by this storage engine"""
        pass



#!/usr/bin/env python
# encoding: utf-8
"""
decorators.py - authentication related decorators

Created by Maximillian Dornseif on 2009-03-15.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

import doctest
import sys
import django.contrib.auth
from functools import wraps, update_wrapper

from django.contrib.auth import authenticate, login
from django.http import HttpResponse


def get_credentials(authheader):
    r"""Extract HTTP-Basic Auth Credentials and return username, password).
    
    >>> get_credentials('basic dGVzdHVzZXI6dGVzdHBhc3N3b3Jk\n')
    ('testuser', 'testpassword')
    
    """
    
    if not authheader:
        return None, None
    
    authmeth, auth = authheader.split(' ', 1)
    if authmeth.lower() != 'basic':
        # return HttpResponse('unknown authentication method', status=406)
        return None, None
    
    auth = auth.strip().decode('base64')
    username, password = auth.split(':', 1)
    return username, password
    

def require_login(func):
    """This decorator logs in a user via HTTP-Auth.
    
    It dos NOT require HTTP Auth is the user already has an active session.
    If the users send via HTTP-Auth differs from the user in the active session, the Session is
    dumped and login with the cedentials provided via HTTP-Auth is retried.
    """
    
    @wraps(func)
    def _decorator(request, *args, **kwargs):
        # no docstring since docstring will be overwritten by the decorated function
        username, password = get_credentials(request.META.get('HTTP_AUTHORIZATION'))
        
        if request.user.is_authenticated():
            if username and username not in [request.user.username, request.user.email]:
                # username given in HTTP-Auth differs from username in session. This should never happen.
                # logout and force reauthentication.
                django.contrib.auth.logout(request)
                response = HttpResponse('Authentication required', status=401)
                response['WWW-Authenticate'] = 'Basic realm="Internal"'
                return response
            else:
                if not request.user.is_active:
                    return HttpResponse('disabled account', status=403)
                # user already is logged in, just proceed to the wrapped function
                return func(request, *args, **kwargs)
        
        # user is not logged in and didn't provide authentication credentials - request authentication
        if 'HTTP_AUTHORIZATION' not in request.META:
            response = HttpResponse('Authentication required', status=401)
            response['WWW-Authenticate'] = 'Basic realm="Internal"'
            return response
            
        if not username:
            return HttpResponse('unknown authentication method', status=406)
        
        # Ask Django to do authentication for us.
        user = authenticate(username=username, password=password)
        
        if user:
            if not user.is_active:
                return HttpResponse('disabled account', status=403)
            else:
                login(request, user)
                return func(request, *args, **kwargs)
        else:
            # Django didn't find anything, ask for authentication again.
            response = HttpResponse('invalid login', status=401)
            response['WWW-Authenticate'] = 'Basic realm="Internal"'
            return response
    
    return _decorator


def deny_unless_user_passes_test(test_func):
    """Decorator that thecks if a user passes a test and if not renders a failure message.
    
    The decorator assumes the user is already authenticated. You might want to use it in conjunction
    with require_login.
    
    Similar to django.contrib.auth.decorators.user_passes_test but without redirection.
    """
    
    def wrap(func):
        print "WWW", func, test_func
        # @wraps(func)
        def _decorator(request, *args, **kwargs):
            print "deco!", test_func, request.user, test_func(request.user)
            if not test_func(request.user):
                return HttpResponse('Access denied / Zugriff verweigert (%s:%s)' % (request.user.id,
                            request.user.username), status=403)
            return func(request, *args, **kwargs)
        return _decorator
    return wrap

if __name__ == '__main__':
    failure_count, test_count = doctest.testmod()
    sys.exit(failure_count)



# coding: utf-8

from django.conf.urls.defaults import *

urlpatterns = patterns('',
    # Always provide the auth system login and logout views
    (r'^accounts/login/$', 'django.contrib.auth.views.login', {'template_name': 'login.html'}),
    (r'^accounts/logout/$', 'django.contrib.auth.views.logout'),
    
    # test urlconf for middleware tests
    (r'^middleware/', include('regressiontests.middleware.urls')),
)

urlpatterns = patterns('tests.regressiontests.auth_decorators.views',
    (r'^auth_decorators/require_login/$', 'require_login_view'))



"""huDjango contains a collection of small extensions the the Django Web framework."""

import codecs
from setuptools import setup, find_packages

hubarcode = setup(name='huDjango',
      maintainer='Maximillian Dornseif',
      maintainer_email='md@hudora.de',
      url='http://github.com/hudora/huDjango',
      version='0.91p5',
      description='various snippets for use with Django',
      long_description=codecs.open('README.textile', "r", "utf-8").read(),
      classifiers=['License :: OSI Approved :: BSD License',
                   'Intended Audience :: Developers',
                   'Programming Language :: Python'],
      zip_safe=False,
      packages=find_packages(),
      install_requires=['Django>=1.0.2', 'huimages>=1.01', 'django-storages',
                        'textile', 'markdown', 'xlwt',
                        'docutils', 'CouchDB>=0.6.1', 'python-memcached'],
)



# encoding: utf-8

from optparse import make_option
from django.core.management.base import BaseCommand
from uuid import uuid4
import couchdb
import re
import os
import json

import couchdb.client


class CouchDBCommand(BaseCommand):
    
    option_list = BaseCommand.option_list + (
        make_option('--server', '-s', default='http://localhost:5984/', help='couchdb server to use [default: %default]'),
        #make_option('--database', '-d', help='couchdb database to use'),
    )

    def _is_json_filename(self, filename):
        return os.path.splitext(filename)[-1] == ".json"

    def _server(self, servername):
        server = couchdb.client.Server(servername)
        return server

    def _db(self, options):
        db = self._server(options['server'])[options['database']]
        return db
    
    def _documents_path(self, options):
        path = os.path.join(options['path'], 'documents')
        return path

    def load_json_document(self, filepath):
        # zuerst laden wir das original Dokument
        try: 
            doc = json.load(open(filepath))
        except Exception:
            raise Exception("unable to parse json document '%s'!" % filepath)

        # wenn wir die ID geaendert haben speichern wir das Dokument noch
        if not '_id' in doc:
            doc_id = uuid4().hex
            fh = open(filepath, 'w')
            json.dump(doc, fh, indent=4, sort_keys=True)
            fh.close()
        
        # und jetzt das Document mit seiner ID zurueckliefern
        return doc['_id'], doc



#!/usr/bin/env python
# encoding: utf-8
"""
tests.py

Created by Maximillian Dornseif on 2009-08-17.
Copyright (c) 2009 HUDORA. All rights reserved.
"""

from datetime import datetime
from django.test import TestCase
from regressiontests.serializers.models import Place
from hudjango.serializers import queryset_to_xls

class SerializerTests(TestCase):
    def test_serializer_list(self):
        place1 = Place.objects.create(name="Dempsey's", address="623 Vermont St",
                                      last_visit=datetime(2008, 7, 6, 5, 4, 3))
        place2 = Place.objects.create(name="Eich", address=u"An dür Eich 1",
                                      last_visit=datetime(2008, 7, 6, 5, 4, 3))
        xlsdata = queryset_to_xls([place1, place2])
        self.assertEqual(xlsdata[:4], '\xd0\xcf\x11\xe0')



__test__ = {'API_TESTS': """
# >>> from auth.backends import EmailBackend
# >>> backend = EmailBackend()
# >>> repr(backend.authenticate(username='foo', password='testpw'))
# 'None'

# >>> from django.contrib.auth.models import User, AnonymousUser
# >>> u = User.objects.create_user('authtestuser', 'authtest@example.com', 'testpw')
# >>> u.save()
# 
# >>> repr(backend.authenticate(username='foo', password='testpw'))
# 'None'
# >>> repr(backend.authenticate(username='authtest@example.com', password='wrong'))
# 'None'
# >>> repr(backend.authenticate(username='authtestuser', password='wrong'))
# 'None'
# 
# 
# We can login with username
# >>> backend.authenticate(username='authtestuser', password='testpw').username
# u'authtestuser'
# 
# And we can log in with email adress
# >>> backend.authenticate(username='authtest@example.com', password='testpw').username
# u'authtestuser'
# 
# If a E-Mail adress appears twice in the database login by that E-Mail address is disabled.
# >>> u2 = User.objects.create_user('authtestuser2', 'authtest@example.com', 'testpw')
# >>> u2.save()
# >>> repr(backend.authenticate(username='authtest@example.com', password='testpw'))
# 'None'
# 
# But we still can log in with the username.
# >>> backend.authenticate(username='authtestuser', password='testpw').username
# u'authtestuser'
# >>> backend.authenticate(username='authtestuser2', password='testpw').username
# u'authtestuser2'
# 
# 
# 
# This LDAP tests actually need an LDAP-Server to run.
# >>> from auth.backends import ZimbraBackend
# >>> backend = ZimbraBackend()
# >>> repr(backend.authenticate(username='foo', password='testpw'))
# 'None'
# >>> len(User.objects.filter(email='md@hudora.de'))
# 0
# >>> repr(backend.authenticate(username='md@hudora.de', password='testpw'))
# 'None'
# >>> backend.authenticate(username='md@hudora.de', password='GEHEIM').username
# 'm.dornseif'
# 
# Does it work twice? (second time we get back an unicode object ...)
# >>> backend.authenticate(username='md@hudora.de', password='GEHEIM').username
# u'm.dornseif'
# 

"""}
